/*
 * Copyright (c) 2016, Freescale Semiconductor, Inc.
 * Copyright 2017 NXP
 * All rights reserved.
 *
 * 
 * SPDX-License-Identifier: BSD-3-Clause
 */


#include "fsl_lin.h"
#include "lin_cfg.h"

/* definition and initialization of signal array */
uint8_t  g_lin_frame_data_buffer[LIN_FRAME_BUF_SIZE] =
{
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
};

/* definition and initialization of signal array */
uint8_t g_lin_flag_handle_tbl[LIN_FLAG_BUF_SIZE] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,0x00 };

/**********************************  Frame table **********************************/
static const lin_frame_struct lin_frame_tbl[LIN_NUM_OF_FRMS] = {

    { LIN_FRM_UNCD, 2U, LIN_RES_PUB, 0U, 0U, 1U, 3U, 0U }  // for commands: gCommand_ToggleLedRed_c, gCommand_ToggleLedGreen_c, gCommand_ToggleLedBlue_c, gCommand_GetLed_c: 2 bytes payload (command and sequence number)

    ,{ LIN_FRM_UNCD, 2U, LIN_RES_PUB, 2U, 0U, 1U, 3U, 0U } // for command: gCommand_GetTemperature_c: 2 bytes payload (command and sequence number)
        
    ,{ LIN_FRM_UNCD, 2U, LIN_RES_SUB, 4U, 1U, 1U, 3U, 0U } // for commands: gCommand_SW2_c, gCommand_SW3_c: 2 bytes payload (command and sequence number)
    
    ,{ LIN_FRM_UNCD, 3U, LIN_RES_SUB, 6U, 1U, 1U, 3U, 0U } // for command: gCommand_SendTemperature_c: 3 bytes payload (command, sequence number and temperature)
    
    ,{ LIN_FRM_UNCD, 5U, LIN_RES_SUB, 9U, 1U, 1U, 3U, 0U } // for command: gCommand_SendLedStatus_c: 5 bytes payload (command, sequence number, led red status, led green status and led blue status)
    
    ,{ LIN_FRM_UNCD, 2U, LIN_RES_SUB, 14U, 1U, 1U, 3U, 0U } // for command: gCommand_SendSucessStatus_c: 2 bytes payload (command, sequence number and temperature)
        
};

static uint8_t LI0_lin_configuration_RAM[LI0_LIN_SIZE_OF_CFG]= {0x00, gID_Leds_c, gID_GetTemp_c, gID_SW2_SW3_c, gID_SendTemperature_c, gID_SendLedStatus_c, gID_SendSucessStatus_c}; 
const uint16_t  LI0_lin_configuration_ROM[LI0_LIN_SIZE_OF_CFG]= {0x0000, gID_Leds_c, gID_GetTemp_c, gID_SW2_SW3_c, gID_SendTemperature_c, gID_SendLedStatus_c, gID_SendSucessStatus_c}; 


/****************************LIN interface configuration ****************************/
const lin_protocol_user_config_t g_lin_protocol_user_cfg_array[LIN_NUM_OF_IFCS] = {

    /* Interface_name = LI0 */
    {
        .protocol_version = LIN_PROTOCOL_21,         /*lin_protocol_version */
        .language_version = LIN_PROTOCOL_21,         /*lin_language_version */

        .number_of_configurable_frames = 7,                            /*  num_of_frames */
        .frame_start = 0,                              /*  frame_start */
        .frame_tbl_ptr = lin_frame_tbl,                                          /*  frame_tbl */
        .list_identifiers_ROM_ptr = LI0_lin_configuration_ROM,       /*  *configuration_ROM */
        .list_identifiers_RAM_ptr = LI0_lin_configuration_RAM,       /*  *configuration_RAM */
        .max_idle_timeout_cnt =  10000,     /* Max Idle Timeout Count */
        .max_message_length = 6             /* Max message length */
    }
};


lin_protocol_state_t g_lin_protocol_state_array[LIN_NUM_OF_IFCS] = {

    /* Interface_name = LI0 */
    {
        .successful_transfer = 0U,
        .num_of_processed_frame = 0U,
        .num_of_successfull_frame = 0U,
        .diagnostic_mode = DIAG_INTERLEAVE_MODE,
        .error_in_response = 0U,
        .timeout_in_response = 0U,
        .transmit_error_resp_sig_flg = 0U,
        .go_to_sleep_flg = 0U,
        .next_transmit_tick = 0U,
        .save_config_flg = 0U,
        .event_trigger_collision_flg = 0U
    }
};
