/*
 * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * Copyright 2016-2018 NXP
 * All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#include "fsl_debug_console.h"
#include "fsl_flexcan.h"
#include "board.h"

#include "fsl_device_registers.h"
#include "fsl_common.h"
#include "pin_mux.h"
#include "clock_config.h"

#include "SecLib.h"

/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define EXAMPLE_CAN CAN0
#define EXAMPLE_CAN_CLKSRC kCLOCK_BusClk
#define EXAMPLE_CAN_CLK_FREQ CLOCK_GetFreq(kCLOCK_BusClk)
#define USE_CANFD (1)
/* 
 *    DWORD_IN_MB    DLC    BYTES_IN_MB             Maximum MBs
 *    2              8      kFLEXCAN_8BperMB        32
 *    4              10     kFLEXCAN_16BperMB       21
 *    8              13     kFLEXCAN_32BperMB       12
 *    16             15     kFLEXCAN_64BperMB       7
 *
 * Dword in each message buffer, Length of data in bytes, Payload size must align,
 * and the Message Buffers are limited corresponding to each payload configuration:
 */
#define DWORD_IN_MB (4)
#define DLC (10)
#define BYTES_IN_MB kFLEXCAN_16BperMB
#define RX_MESSAGE_BUFFER_NUM (9)
#define TX_MESSAGE_BUFFER_NUM (8)


/*******************************************************************************
 * Prototypes
 ******************************************************************************/

/*******************************************************************************
 * Variables
 ******************************************************************************/
flexcan_handle_t flexcanHandle;
volatile bool txComplete = false;
volatile bool rxComplete = false;
flexcan_mb_transfer_t txXfer, rxXfer;
#if (defined(USE_CANFD) && USE_CANFD)
flexcan_fd_frame_t frame;
#else
flexcan_frame_t frame;
#endif
uint32_t txIdentifier;
uint32_t rxIdentifier;

uint8_t key128[128/8] = { 0x55, 0x6c, 0x74, 0x72,  
                          0x61, 0x53, 0x65, 0x63, 
	                  0x61, 0x73, 0x73, 0x77, 
                          0x6f, 0x72, 0x64, 0x31 };     /*AES128 key*/

void app_rx_can(void);

void app_tx_can (uint8_t databyte0, uint8_t databyte1, uint8_t databyte2, uint8_t databyte3);

extern void APP_ProcessCommand_Can(uint8_t command);

/*******************************************************************************
 * Code
 ******************************************************************************/
/*!
 * @brief FlexCAN Call Back function
 */
static void flexcan_callback(CAN_Type *base, flexcan_handle_t *handle, status_t status, uint32_t result, void *userData)
{
  uint8_t canframe_temp [16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  uint8_t canframe_decrypted_temp [16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  
    switch (status)
    {
        case kStatus_FLEXCAN_RxIdle:
            if (RX_MESSAGE_BUFFER_NUM == result)
            {
                rxComplete = true;

                canframe_temp [0] = frame.dataByte0;
                canframe_temp [1] = frame.dataByte1;
                canframe_temp [2] = frame.dataByte2;
                canframe_temp [3] = frame.dataByte3;
                canframe_temp [4] = frame.dataByte4;
                canframe_temp [5] = frame.dataByte5;
                canframe_temp [6] = frame.dataByte6;
                canframe_temp [7] = frame.dataByte7;
                canframe_temp [8] = frame.dataByte8;
                canframe_temp [9] = frame.dataByte9;
                canframe_temp [10] = frame.dataByte10;
                canframe_temp [11] = frame.dataByte11;
                canframe_temp [12] = frame.dataByte12;
                canframe_temp [13] = frame.dataByte13;
                canframe_temp [14] = frame.dataByte14;
                canframe_temp [15] = frame.dataByte15;
                
                AES_128_Decrypt(canframe_temp, key128, canframe_decrypted_temp);
                
                APP_ProcessCommand_Can(canframe_decrypted_temp[0]);
                
                app_rx_can();  

            }
            break;

        case kStatus_FLEXCAN_TxIdle:
            if (TX_MESSAGE_BUFFER_NUM == result)
            {
                txComplete = true;
            }
            break;

        default:
            break;
    }
}

/*!
 * @brief Main function
 */
void can_demo_init(void)
{
    flexcan_config_t flexcanConfig;
    flexcan_rx_mb_config_t mbConfig;
  
    txIdentifier = 0x321;
    rxIdentifier = 0x123;
    
        /* Initialize board hardware. */
     CAN_InitPins();
    
     /* Get FlexCAN module default Configuration. */
     /*
     * flexcanConfig.clkSrc = kFLEXCAN_ClkSrcOsc;
     * flexcanConfig.baudRate = 1000000U;
     * flexcanConfig.maxMbNum = 16;
     * flexcanConfig.enableLoopBack = false;
     * flexcanConfig.enableSelfWakeup = false;
     * flexcanConfig.enableIndividMask = false;
     * flexcanConfig.enableDoze = false;
     * flexcanConfig.timingConfig = timingConfig;
     */
    FLEXCAN_GetDefaultConfig(&flexcanConfig);

    /* Init FlexCAN module. */
#if (!defined(FSL_FEATURE_FLEXCAN_SUPPORT_ENGINE_CLK_SEL_REMOVE)) || !FSL_FEATURE_FLEXCAN_SUPPORT_ENGINE_CLK_SEL_REMOVE
    flexcanConfig.clkSrc = kFLEXCAN_ClkSrcPeri;
#endif /* FSL_FEATURE_FLEXCAN_SUPPORT_ENGINE_CLK_SEL_REMOVE */
    /* If special quantum setting is needed, set the timing parameters. */
#if (defined(SET_CAN_QUANTUM) && SET_CAN_QUANTUM)
    flexcanConfig.timingConfig.phaseSeg1 = PSEG1;
    flexcanConfig.timingConfig.phaseSeg2 = PSEG2;
    flexcanConfig.timingConfig.propSeg = PROPSEG;
#if (defined(FSL_FEATURE_FLEXCAN_HAS_FLEXIBLE_DATA_RATE) && FSL_FEATURE_FLEXCAN_HAS_FLEXIBLE_DATA_RATE)
    flexcanConfig.timingConfig.fphaseSeg1 = FPSEG1;
    flexcanConfig.timingConfig.fphaseSeg2 = FPSEG2;
    flexcanConfig.timingConfig.fpropSeg = FPROPSEG;
#endif
#endif

#if (defined(USE_CANFD) && USE_CANFD)
    FLEXCAN_FDInit(EXAMPLE_CAN, &flexcanConfig, EXAMPLE_CAN_CLK_FREQ, BYTES_IN_MB, false);
#else
    FLEXCAN_Init(EXAMPLE_CAN, &flexcanConfig, EXAMPLE_CAN_CLK_FREQ);
#endif

    /* Create FlexCAN handle structure and set call back function. */
    FLEXCAN_TransferCreateHandle(EXAMPLE_CAN, &flexcanHandle, flexcan_callback, NULL);

    /* Set Rx Masking mechanism. */
    FLEXCAN_SetRxMbGlobalMask(EXAMPLE_CAN, FLEXCAN_RX_MB_STD_MASK(rxIdentifier, 0, 0));

    /* Setup Rx Message Buffer. */
    mbConfig.format = kFLEXCAN_FrameFormatStandard;
    mbConfig.type = kFLEXCAN_FrameTypeData;
    mbConfig.id = FLEXCAN_ID_STD(rxIdentifier);
#if (defined(USE_CANFD) && USE_CANFD)
    FLEXCAN_SetFDRxMbConfig(EXAMPLE_CAN, RX_MESSAGE_BUFFER_NUM, &mbConfig, true);
#else
    FLEXCAN_SetRxMbConfig(EXAMPLE_CAN, RX_MESSAGE_BUFFER_NUM, &mbConfig, true);
#endif

    /* Setup Tx Message Buffer. */
#if (defined(USE_CANFD) && USE_CANFD)
    FLEXCAN_SetFDTxMbConfig(EXAMPLE_CAN, TX_MESSAGE_BUFFER_NUM, true);
#else
    FLEXCAN_SetTxMbConfig(EXAMPLE_CAN, TX_MESSAGE_BUFFER_NUM, true);
#endif

   app_rx_can();    
   
}


void app_rx_can(void){

            /* Start receive data through Rx Message Buffer. */
            rxXfer.mbIdx = RX_MESSAGE_BUFFER_NUM;
#if (defined(USE_CANFD) && USE_CANFD)
            rxXfer.framefd = &frame;
            FLEXCAN_TransferFDReceiveNonBlocking(EXAMPLE_CAN, &flexcanHandle, &rxXfer);
#else
            rxXfer.frame = &frame;
            FLEXCAN_TransferReceiveNonBlocking(EXAMPLE_CAN, &flexcanHandle, &rxXfer);
#endif


}



void app_tx_can(uint8_t databyte0, uint8_t databyte1, uint8_t databyte2, uint8_t databyte3){


  uint8_t canframe_temp [16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  uint8_t canframe_encrypted_temp [16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  
  canframe_temp[0]=databyte0;
  canframe_temp[1]=databyte1;
  canframe_temp[2]=databyte2;
  canframe_temp[3]=databyte3;
  
  AES_128_Encrypt(canframe_temp, key128, canframe_encrypted_temp); 
  
    frame.dataByte0 = canframe_encrypted_temp[0];  
    frame.dataByte1 = canframe_encrypted_temp[1];  
    frame.dataByte2 = canframe_encrypted_temp[2];  
    frame.dataByte3 = canframe_encrypted_temp[3]; 
  
    frame.dataByte4 = canframe_encrypted_temp[4];  
    frame.dataByte5 = canframe_encrypted_temp[5];  
    frame.dataByte6 = canframe_encrypted_temp[6];  
    frame.dataByte7 = canframe_encrypted_temp[7]; 
    
    frame.dataByte8 = canframe_encrypted_temp[8];  
    frame.dataByte9 = canframe_encrypted_temp[9];  
    frame.dataByte10 = canframe_encrypted_temp[10];  
    frame.dataByte11 = canframe_encrypted_temp[11]; 
  
    frame.dataByte12 = canframe_encrypted_temp[12];  
    frame.dataByte13 = canframe_encrypted_temp[13];  
    frame.dataByte14 = canframe_encrypted_temp[14];  
    frame.dataByte15 = canframe_encrypted_temp[15];    

    frame.id = FLEXCAN_ID_STD(txIdentifier);
    frame.format = kFLEXCAN_FrameFormatStandard;
    frame.type = kFLEXCAN_FrameTypeData;
    frame.length = 15; //@lb before 1
    txXfer.mbIdx = TX_MESSAGE_BUFFER_NUM;

	txXfer.mbIdx = TX_MESSAGE_BUFFER_NUM;
	#if (defined(USE_CANFD) && USE_CANFD)
	            txXfer.framefd = &frame;
	            FLEXCAN_TransferFDSendNonBlocking(EXAMPLE_CAN, &flexcanHandle, &txXfer);
	#else
	            txXfer.frame = &frame;
	            FLEXCAN_TransferSendNonBlocking(EXAMPLE_CAN, &flexcanHandle, &txXfer);
	#endif



}